<?xml version="1.0" encoding="US-ASCII"?>
<!--
     Render a CIQ context/value association file to a simple HTML presentation.
     This provides the basic mechanics so that users can embellish on this
     presentation for their own purposes.

     This stylesheet accesses the genericode2html.xsl stylesheet that is
     assumed to be in the same directory.

     To enable a CIQ context/value association file to be rendered in a 
     browser, add the following processing instruction to the start of 
     the file (changing the href= as required to point to the stylesheet):

     <?xml-stylesheet type="text/xsl" href="Crane-assoc2html.xsl"?>

     When in a Windows interface, drag the association file from the Windows
     Explorer to Mozilla Firefox or Internet Explorer to engage the stylesheet
     presentation.

     When creating standalone HTML from the XML, two examples of using Saxon
     are as follows, the first explicitly referencing the stylesheet and the
     second looking for the embedded stylesheet association:

       java -jar saxon.jar -o file.html file.cva Crane-assoc2html.xsl
       java -jar saxon.jar -o file.html -a file.cva

     Note that it is not necessary for any referenced value list files to have
     an embedded stylesheet when using a stylesheet with a UBL context/value
     association file.

     Implementation note: this version does not attempt to detect duplicate
     presentations of the same code list from multiple references either
     within one context/value file or included context/value files.

 $Id: Crane-assoc2html.xsl,v 1.7 2007/02/10 02:16:12 G. Ken Holman Exp $

 Copyright (C) - Crane Softwrights Ltd.
               - http://www.CraneSoftwrights.com/links/res-dev.htm
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 - Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 - The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 Note: for your reference, the above is the "Modified BSD license", this text
     was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<!-- xmlns:a="urn:oasis:names:tc:ubl:schema:Value-List-Constraints-0.8" -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
             xmlns:a="http://docs.oasis-open.org/codelist/ns/ContextValueAssociation/1.0/"
                xmlns="http://www.w3.org/TR/REC-html40"
                xmlns:x="http://www.w3.org/TR/REC-html40"
                version="1.0">

<xsl:import href="Crane-genericode2html.xsl"/>

<xsl:key name="a:ids" match="*[@xml:id]" use="@xml:id"/>

<!--========================================================================-->
<!--all messages separated so they can be overridden in another language-->

<xsl:variable name="a:titlePrefix">Value list constraints: </xsl:variable>
<xsl:variable name="a:badDocElemTitle">Unexpected XML document for stylesheet</xsl:variable>
<xsl:variable name="a:noTitle">Untitled association file</xsl:variable>
<xsl:variable name="a:fileNotFound">(file not found)</xsl:variable>
<xsl:variable name="a:valueLists">Value Lists</xsl:variable>
<xsl:variable name="a:noValueLists">(no lists defined)</xsl:variable>
<xsl:variable name="a:contexts">Document contexts</xsl:variable>
<xsl:variable name="a:noContexts">(no document contexts defined)</xsl:variable>
<xsl:variable name="a:badListId">
  <font color="red">
    <b>(error: a list with this identifier is not defined)</b>
  </font>
</xsl:variable>
<xsl:template name="a:badDocElem">
  <xsl:param name="ns"/>
  <p>
    Unexpected XML document for <samp>Crane-assoc2html.xsl</samp> stylesheet.
  </p>
  <p>
    The document element is expected to be named 
    <samp>ValueListConstraints</samp> in the
    <samp><xsl:value-of select="$ns"/></samp>
    namespace, otherwise this message is displayed.
  </p>
  <p>
    What was found in this document is the document element named
    <samp><xsl:value-of select="name(.)"/></samp> in the
    <samp><xsl:value-of select="namespace-uri(.)"/></samp> namespace.
  </p>
</xsl:template>

<!--========================================================================-->
<!--wrap processing of document element so that document element can be
    processed on its own in another context-->

<xsl:template match="/">
  <!--all HTML documents start the same way-->
  <html>
    <head>
      <xsl:choose>
        <xsl:when test="a:ValueListConstraints">
          <!--document is what we are expecting-->
          <title>
            <xsl:choose>
              <xsl:when test="a:ValueListConstraints/a:Title">
                <xsl:value-of select="a:ValueListConstraints/a:Title"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:copy-of select="$a:titlePrefix"/>
                name="<xsl:value-of select="*/@name"/>"
                id="<xsl:value-of select="*/@id"/>"
              </xsl:otherwise>
            </xsl:choose>
          </title>
        </xsl:when>
        <xsl:otherwise>
          <!--this is an error situation-->
          <title><xsl:copy-of select="$a:badDocElemTitle"/></title>
        </xsl:otherwise>
      </xsl:choose>
    </head>
    <body>
      <!--present the association files first-->
      <xsl:apply-templates mode="a:assoc2html" select="document('',.)/*"/>
         <!--the above select may look strange, but it prevents a distinction
             between the source document node and the node of the document
             opened up relative to the source; this is necessary for the
             detection of infinite loops-->
      <!--present the list files next-->
      <xsl:apply-templates mode="a:lists" select="document('',.)/*"/>
    </body>
  </html>
</xsl:template>

<xsl:template match="/*" mode="a:assoc2html">
  <!--the document being asked to be formatted is not as expected-->
  <xsl:call-template name="a:badDocElem">
    <xsl:with-param name="ns" select="string(document('')/*/namespace::a)"/>
  </xsl:call-template>
</xsl:template>

<!--========================================================================-->
<!--main without detail, but including infinite loop detection-->

<xsl:template match="/a:ValueListConstraints" mode="a:assoc2html" priority="2">
  <xsl:param name="main"/>
  <xsl:param name="loop-identifiers" select="' '"/>
  <xsl:if test="not( contains( $loop-identifiers, 
                                concat(' ',generate-id(.),' ' ) ) )">
    <!--then not in an infinite loop-->
    <xsl:if test="normalize-space($loop-identifiers)">
      <!--break between files with a horizontal rule-->
      <hr/>
    </xsl:if>
    <xsl:variable name="new-loop-identifiers"
                  select="concat($loop-identifiers,generate-id(.),' ' )"/>
    <!--anchor the title of the file-->
    <a name="{generate-id(.)}">
      <xsl:choose>
        <xsl:when test="a:Title">
          <xsl:apply-templates select="a:Title"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="$a:noTitle"/>
        </xsl:otherwise>
      </xsl:choose>
    </a>
    <!--subtitle with identification information-->
    <h4>
      <samp>name="<xsl:value-of select="@name"/>"</samp>
      <xsl:text> </xsl:text>
      <samp>id="<xsl:value-of select="@id"/>"</samp>
    </h4>
    <xsl:apply-templates select="a:Identification | a:Description"/>
    <!--make references to included files-->
    <xsl:if test="a:Include">
      <h3>Includes referenced</h3>
      <xsl:for-each select="a:Include">
        <xsl:variable name="uri" select="@uri"/>
        <p>
          <samp>
            <xsl:text>uri="</xsl:text>
            <xsl:for-each select="document($uri,.)/*">
              <a href="#{generate-id(.)}"><xsl:value-of select="$uri"/></a>
            </xsl:for-each>
            <xsl:if test="not(document($uri,.))">
              <xsl:value-of select="$uri"/>
            </xsl:if>
            <xsl:text>"</xsl:text>
          </samp>
          <xsl:if test="not(document($uri,.))">
            <xsl:copy-of select="$a:fileNotFound"/>
          </xsl:if>
        </p>
      </xsl:for-each>
    </xsl:if>
    <!--render the information in the current file-->
    <xsl:apply-templates select="a:ValueLists | a:Contexts"/>
    <!--render the information for included files-->
    <xsl:if test="a:Include">
      <xsl:for-each select="a:Include">
        <xsl:variable name="uri" select="@uri"/>
        <xsl:choose>
          <xsl:when test="not(document($uri,.))">
            <p>
              <samp>
                <xsl:text/>uri="<xsl:value-of select="$uri"/>"<xsl:text/>
              </samp>
              <xsl:copy-of select="$a:fileNotFound"/>
            </p>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="document($uri,.)/*" 
                                 mode="a:assoc2html">
              <xsl:with-param name="loop-identifiers" 
                              select="$new-loop-identifiers"/>
            </xsl:apply-templates>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:if>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<!--context vocabulary-->

<!--titles, identification and description-->
<xsl:template match="a:Title">
  <h2><xsl:apply-templates/></h2>
</xsl:template>

<xsl:template match="a:Identification | a:Description">
  <xsl:apply-templates/>
</xsl:template>

<!--value lists-->
<xsl:template match="a:ValueLists">
  <h3><xsl:copy-of select="$a:valueLists"/></h3>
  <xsl:if test="not(a:ValueList)">
    <p><i><xsl:copy-of select="$a:noValueLists"/></i></p>
  </xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="a:ValueList">
  <h4>
    <!--identify the value list-->
    <a name="{generate-id(.)}">
      <samp>xml:id="<xsl:value-of select="@xml:id"/>"</samp>
      <xsl:text> </xsl:text>
      <samp>uri="<xsl:text/>
        <a href="#{generate-id(document(@uri,.)/*)}">
          <xsl:value-of select="@uri"/>
        </a>
        <xsl:text/>"</samp>
    </a>
  </h4>
  <xsl:if test="string(.)">
    <!--documentation-->
    <blockquote>
      <xsl:apply-templates/>
    </blockquote>
  </xsl:if>
</xsl:template>

<xsl:template match="a:MetaData">
  <!--overriding metadata-->
  <table>
    <xsl:apply-templates mode="a:metadata"/>
  </table>
</xsl:template>

<xsl:template match="*" mode="a:metadata">
  <tr>
    <td valign="top"><samp><xsl:value-of select="local-name(.)"/></samp></td>
    <td valign="top"><samp>=</samp></td>
    <td valign="top"><samp><xsl:value-of select="."/></samp></td>
  </tr>
</xsl:template>

<!--contexts-->
<xsl:template match="a:Contexts">
  <h3><xsl:copy-of select="$a:contexts"/></h3>
  <xsl:if test="not(a:Context)">
    <p><i><xsl:copy-of select="$a:noContexts"/></i></p>
  </xsl:if>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="a:Context">
  <!--identify the context-->
  <h4>
    <samp>item="<xsl:value-of select="@item"/>"</samp>
    <xsl:for-each select="@context">
      <xsl:text> </xsl:text>
      <samp>context="<xsl:value-of select="."/>"</samp>
    </xsl:for-each>
    <xsl:for-each select="@xpath">
      <xsl:text> </xsl:text>
      <samp>xpath="<xsl:value-of select="."/>"</samp>
    </xsl:for-each>
  </h4>
  <!--qualify the context-->
  <ul xml:space="preserve">
    <xsl:call-template name="values"/>
    <xsl:apply-templates select="a:Message"/>
  </ul>
  <!--documentation-->
  <xsl:if test="string(.)">
    <blockquote>
      <xsl:apply-templates select="*[not(self::a:Message)]"/>
    </blockquote>
  </xsl:if>
</xsl:template>

<!--recursive reporting of the lists of values-->
<xsl:template name="values">
  <xsl:param name="lists" select="concat(normalize-space(@values),' ' )"/>
  <xsl:variable name="list" select="substring-before($lists,' ')"/>
  <xsl:if test="$list">
    <xsl:if test="not(key('a:ids',$list))">
      <!--report missing information for the next linked list of values-->
      <li>
        <samp>xml:id="<xsl:value-of select="$list"/>"
        <xsl:copy-of select="$a:badListId"/>
        </samp>
      </li>
    </xsl:if>
    <xsl:for-each select="key('a:ids',$list)">
      <li>
      <!--report information for the next linked list of values-->
        <samp>xml:id="<xsl:text/>
          <a href="#{generate-id(.)}">
            <xsl:value-of select="@xml:id"/></a>"</samp>
      </li>
    </xsl:for-each>
    <!--report the next value-->
    <xsl:call-template name="values">
      <xsl:with-param name="lists" select="substring-after($lists,' ')"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!--a customized Schematron message-->
<xsl:template match="a:Message">
  <li>
    <samp>&lt;Message></samp>
    <xsl:apply-templates select="node()" mode="expose"/>
  </li>
</xsl:template>

<!--expose the markup in monospaced text-->
<xsl:template match="*" mode="expose">
  <!--start tag-->
  <samp>
    <xsl:text/>&lt;<xsl:value-of select="name(.)"/>
    <xsl:for-each select="@*">
      <xsl:value-of select="concat(' ',name(.),'=&#34;',.,'&#34;')"/>
    </xsl:for-each>
    <!--check for empty tag-->
    <xsl:if test="not(node())">/</xsl:if>
    <xsl:text>></xsl:text>
  </samp>
  <xsl:if test="node()">
    <!--content-->
    <xsl:apply-templates mode="expose"/>
    <!--end tag-->
    <samp>
      <xsl:text/>&lt;/<xsl:value-of select="name(.)"/>><xsl:text/>
    </samp>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<!--finding genericode files-->

<!--this needs to be updated to prevent the presentation of duplicates-->
<xsl:template match="/a:ValueListConstraints" mode="a:lists">
  <xsl:for-each select="a:ValueLists/a:ValueList/@uri">
    <hr/>
    <xsl:apply-templates select="document(.,.)/*">
      <xsl:with-param name="uri" select="."/>
    </xsl:apply-templates>
  </xsl:for-each>
</xsl:template>

<!--anything else in this namespace doesn't trigger any output-->
<xsl:template match="*" mode="a:lists"/>

<!--========================================================================-->
<!--HTML vocabulary-->

<xsl:template match="x:*">
  <xsl:element name="{local-name()}" namespace="{namespace::x}">
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates/>
  </xsl:element>
</xsl:template>

</xsl:stylesheet>
